unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ToolWin, MSWord, MSWPD, MSWBase, MSWRI, Menus,
  MSWOO, MSWFB2, MSPalmDoc, SMWPDF, ImgList, MSW_Sum;

type
  TfrmMain = class(TForm)
    OpenDialog: TOpenDialog;
    MSWordDocument1: TMSWordDocument;
    tbMain: TToolBar;
    btnOpen: TToolButton;
    ImageList: TImageList;
    btnStatistic: TToolButton;
    btnSeparator1: TToolButton;
    btnClose: TToolButton;
    btnAbout: TToolButton;
    btnOrder: TToolButton;
    btnSeparator2: TToolButton;
    pcDocument: TPageControl;
    tsPlainText: TTabSheet;
    memoDocument: TMemo;
    tsMacros: TTabSheet;
    memoMacros: TMemo;
    WordPerfectFile1: TWordPerfectFile;
    WRIFile1: TWRIFile;
    OpenOfficeTextDocument1: TOpenOfficeTextDocument;
    OpenOfficePresentation1: TOpenOfficePresentation;
    OpenOfficeSpreadsheet1: TOpenOfficeSpreadsheet;
    procedure btnOpenClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnOrderClick(Sender: TObject);
    procedure btnStatisticClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure MSWordDocument1ConvertSysChar(Sender: TObject;
      SysChar: TSMWordSysChar; Value: WideString; var Result: WideString);
    procedure MSWordDocument1GetDefaultLanguage(Sender: TObject;
      var DocLang: Word);
  private
    { Private declarations }
    procedure ShowSummaryInfo(si: TSMWordSummaryInformation);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}
{$R winxp.res}

uses ShellAPI, SumInfo;

function GetPlain(lst: TWideStrings): WideString;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to lst.Count-1 do
    Result := Result + lst[i]
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
var
  strExt: string;
//  StartTick, EndTick: Integer;
  docx: TMSWordXDocument;
  pdf: TSMPDFReader;
begin
  if OpenDialog.Execute then
  begin
    strExt := UpperCase(ExtractFileExt(OpenDialog.FileName));
    if (strExt = '.WPD') then
    begin
      WordPerfectFile1.FileName := OpenDialog.FileName;
      WordPerfectFile1.Execute;

      memoDocument.Lines.Text := WordPerfectFile1.PlainText.Text;
    end
    else
    if (strExt = '.WRI') then
    begin
      WRIFile1.FileName := OpenDialog.FileName;
      WRIFile1.Execute;

      memoDocument.Lines.Text := WRIFile1.PlainText.Text;
    end
    else
    if (strExt = '.DOCX') then
    begin
      docx := TMSWordXDocument.Create(Self);
      try
        docx.FileName := OpenDialog.FileName;
//          TextFormating := tfHTML;

        docx.Execute;

        memoDocument.Lines.Text := docx.PlainText.Text;
        ShowSummaryInfo(docx.SummaryInformation)
      finally
        docx.Free
      end
    end
    else
    if (strExt = '.ODT') then
    begin
      OpenOfficeTextDocument1.FileName := OpenDialog.FileName;
      OpenOfficeTextDocument1.Execute;

      memoDocument.Lines.Text := OpenOfficeTextDocument1.PlainText.Text;
    end
    else
    if (strExt = '.ODP') then
    begin
      OpenOfficePresentation1.FileName := OpenDialog.FileName;
      OpenOfficePresentation1.Execute;

      memoDocument.Lines.Text := OpenOfficePresentation1.PlainText.Text;
    end
    else
    if (strExt = '.ODS') then
    begin
      OpenOfficeSpreadsheet1.FileName := OpenDialog.FileName;
      OpenOfficeSpreadsheet1.Execute;

      memoDocument.Lines.Text := OpenOfficeSpreadsheet1.PlainText.Text;
    end
    else
    if (strExt = '.FB2') then
    begin
      with TFictionBook.Create(Self) do
        try
          FileName := OpenDialog.FileName;
//          TextFormating := tfHTML;
          
          Execute;

          memoDocument.Lines.Text := PlainText.Text;
        finally
          Free
        end
    end
    else
    if (strExt = '.PDB') then
    begin
      with TPalmDocFile.Create(Self) do
        try
          FileName := OpenDialog.FileName;
//          TextFormating := tfHTML;

          Execute;

          memoDocument.Lines.Text := PlainText.Text;
        finally
          Free
        end
    end
    else
    if (strExt = '.PDF') then
    begin
      pdf := TSMPDFReader.Create(Self);
      try
        pdf.FileName := OpenDialog.FileName;

        pdf.Execute;

        memoDocument.Lines.Text := pdf.PlainText.Text;
        ShowSummaryInfo(pdf.SummaryInformation)
      finally
        pdf.Free
      end
    end
    else
    begin
      MSWordDocument1.ParseItems := MSWordDocument1.ParseItems - [wpiVBA];
      MSWordDocument1.FileName := OpenDialog.FileName;
//      MSWordDocument1.Text.SaveUnicode := True;

//      StartTick := GetTickCount;
      MSWordDocument1.Execute;
//      EndTick := GetTickCount;
//      Caption := IntToStr(EndTick - StartTick + 1) + ' msec';

//       MSWordDocument1.Text.SaveToFile(ChangeFileExt(OpenDialog.FileName, '.txt'));

      memoDocument.Lines.Text := GetPlain(MSWordDocument1.PlainText);//MSWordDocument1.PlainText.Text;
      memoMacros.Lines.Text := MSWordDocument1.MacroText.Text;
    end;
    btnStatistic.Enabled := True
  end;
end;

function DT2Str(dt: TDateTime): string;
begin
  if dt = 0 then
    Result := ''
  else
    Result := DateToStr(dt)
end;

procedure TfrmMain.ShowSummaryInfo(si: TSMWordSummaryInformation);
begin
  with TfrmSummaryInfo.Create(Application) do
    try
      with lvSummary.Items.Add do
      begin
        Caption := 'FileName';
        SubItems.Add(OpenDialog.FileName)
      end;

      with lvSummary.Items.Add do
      begin
        Caption := 'Title';
        SubItems.Add(si.Title)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'Subject';
        SubItems.Add(si.Subject)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'Author';
        SubItems.Add(si.Author)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'Application';
        SubItems.Add(si.AppName)
      end;

      with lvSummary.Items.Add do
      begin
        Caption := 'Keywords';
        SubItems.Add(si.Keywords)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'Comments';
        SubItems.Add(si.Comments)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'Template';
        SubItems.Add(si.Template)
      end;

      with lvSummary.Items.Add do
      begin
        Caption := 'LastAuthor';
        SubItems.Add(si.LastAuthor)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'Created';
        SubItems.Add(DT2Str(si.Created))
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'LastSaved';
        SubItems.Add(DT2Str(si.LastSaved))
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'LastPrinted';
        SubItems.Add(DT2Str(si.LastPrinted))
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'RevNumber';
        SubItems.Add(si.RevNumber)
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'EditTime (min)';
        SubItems.Add(IntToStr(si.EditTime))
      end;

      with lvSummary.Items.Add do
      begin
        Caption := 'PageCount';
        SubItems.Add(IntToStr(si.PageCount))
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'CharCount';
        SubItems.Add(IntToStr(si.CharCount))
      end;
      with lvSummary.Items.Add do
      begin
        Caption := 'WordCount';
        SubItems.Add(IntToStr(si.WordCount))
      end;

      ShowModal
    finally
      Free
    end;
end;

procedure TfrmMain.btnStatisticClick(Sender: TObject);
begin
  ShowSummaryInfo(MSWordDocument1.SummaryInformation)
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close
end;

type
  LONG  = DWORD;
  {$IFDEF UNICODE}
  LPAWSTR = PWideChar;
  {$ELSE}
  LPAWSTR = PAnsiChar;
  {$ENDIF}

type
  HCRYPTPROV  = ULONG;
  PHCRYPTPROV = ^HCRYPTPROV;
  HCRYPTKEY   = ULONG;
  PHCRYPTKEY  = ^HCRYPTKEY;
  HCRYPTHASH  = ULONG;
  PHCRYPTHASH = ^HCRYPTHASH;

  ALG_ID = ULONG;

function CryptAcquireContextA(phProv       :PHCRYPTPROV;
                              pszContainer :PAnsiChar;
                              pszProvider  :PAnsiChar;
                              dwProvType   :DWORD;
                              dwFlags      :DWORD) :BOOL; stdcall; external ADVAPI32 name 'CryptAcquireContextA';

function CryptAcquireContextW(phProv       :PHCRYPTPROV;
                              pszContainer :PWideChar;
                              pszProvider  :PWideChar;
                              dwProvType   :DWORD;
                              dwFlags      :DWORD) :BOOL ; stdcall; external ADVAPI32 name 'CryptAcquireContextW';

{$IFDEF UNICODE}
function CryptAcquireContext(phProv       :PHCRYPTPROV;
                              pszContainer :PWideChar;
                              pszProvider  :PWideChar;
                              dwProvType   :DWORD;
                              dwFlags      :DWORD) :BOOL ; stdcall; external ADVAPI32 name 'CryptAcquireContextW';
{$ELSE}
function CryptAcquireContext(phProv        :PHCRYPTPROV;
                              pszContainer :LPAWSTR;
                              pszProvider  :LPAWSTR;
                              dwProvType   :DWORD;
                              dwFlags      :DWORD) :BOOL; stdcall; external ADVAPI32 name 'CryptAcquireContextA';
{$ENDIF}

function CryptReleaseContext(hProv   :HCRYPTPROV;
                             dwFlags :DWORD) :BOOL; stdcall; external ADVAPI32 name 'CryptReleaseContext';

function CryptDeriveKey(hProv     :HCRYPTPROV;
                        Algid     :ALG_ID;
                        hBaseData :HCRYPTHASH;
                        dwFlags   :DWORD;
                        phKey     :PHCRYPTKEY) :BOOL; stdcall ; external ADVAPI32 name 'CryptDeriveKey';

function CryptEncrypt(hKey       :HCRYPTKEY;
                      hHash      :HCRYPTHASH;
                      Final      :BOOL;
                      dwFlags    :DWORD;
                      pbData     :PBYTE;
                      pdwDataLen :PDWORD;
                      dwBufLen   :DWORD) :BOOL; stdcall; external ADVAPI32 name 'CryptEncrypt';

function CryptDecrypt(hKey       :HCRYPTKEY;
                      hHash      :HCRYPTHASH;
                      Final      :BOOL;
                      dwFlags    :DWORD;
                      pbData     :PBYTE;
                      pdwDataLen :PDWORD) :BOOL; stdcall; external ADVAPI32 name 'CryptDecrypt';


function CryptCreateHash(hProv   :HCRYPTPROV;
                         Algid   :ALG_ID;
                         hKey    :HCRYPTKEY;
                         dwFlags :DWORD;
                         phHash  :PHCRYPTHASH) :BOOL; stdcall; external ADVAPI32 name 'CryptCreateHash';


function CryptHashData(hHash       :HCRYPTHASH;
                 const pbData      :PBYTE;
                       dwDataLen   :DWORD;
                       dwFlags     :DWORD) :BOOL; stdcall; external ADVAPI32 name 'CryptHashData';

function CryptDestroyHash(hHash :HCRYPTHASH) :BOOL; stdcall; external ADVAPI32 name 'CryptDestroyHash';

const
  CRYPT_VERIFYCONTEXT  = $F0000000;

  PROV_RSA_FULL          = 1;

  ALG_CLASS_HASH         = (4 shl 13);
  ALG_TYPE_ANY           = 0;
  ALG_SID_RC4            = 1;
  ALG_SID_MD5            = 3;

  ALG_CLASS_DATA_ENCRYPT = (3 shl 13);
  ALG_TYPE_STREAM        = (4 shl 9);

  CALG_MD5              = (ALG_CLASS_HASH or ALG_TYPE_ANY or ALG_SID_MD5);
  CALG_RC4              = (ALG_CLASS_DATA_ENCRYPT or ALG_TYPE_STREAM or ALG_SID_RC4);

procedure TfrmMain.btnOrderClick(Sender: TObject);
var
  hProv: HCRYPTPROV;
  hash: HCRYPTHASH;
  key: HCRYPTKEY;

  Buffer: PByte;
  len: dWord;

  password: string;
begin
  password := #$28#$BF#$4E#$5E#$4E#$75#$8A#$41#$64#$00#$4E#$56#$FF#$FA#$01#$08#$2E#$2E#$00#$B6#$D0#$68#$3E#$80#$2F#$0C#$A9#$FE#$64#$53#$69#$7A +
              #$1a#$02#$38#$ee#$fa#$cf#$a9#$8a#$d0#$74#$1f#$f9#$67#$8d#$03#$ed#$69#$62#$b0#$64#$83#$09#$8c#$47#$cd#$22#$bd#$f7#$e8#$f5#$32#$4b +
              #$FF#$FF#$F0#$D4 +
              #$26#$f9#$c0#$df#$07#$7f#$0b#$4a#$a1#$5a#$54#$68#$32#$c7#$fc#$6e;

  {get context for crypt default provider}
  CryptAcquireContext(@hProv, nil, nil, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
  {create hash-object (MD5 algorithm)}
  CryptCreateHash(hProv, CALG_MD5, 0, 0, @hash);
  {get hash from password}
  CryptHashData(hash, @Password[1], Length(Password), 0);
  {create key from hash by RC4 algorithm}
  CryptDeriveKey(hProv, CALG_RC4, hash, 0, @key);
  {destroy hash-object}
  CryptDestroyHash(hash);

  {allocate buffer to read content from source file}
  GetMem(Buffer, 512);
  try
    password := #$17#$46#$57#$6a#$22#$dd#$93#$e6#$37#$42#$33#$a0#$fa#$db#$da#$5d#$b3#$18#$00#$0b#$bd#$fa#$ba#$86#$d6#$b5#$a1#$cb#$80#$44#$4f#$97;
    Move(password[1], Buffer, 32);
    {read content from source file}
//    len := fsIn.Read(Buffer^, 512);

    {decrypt buffer}
    CryptDecrypt(key, 0, True, 0, Buffer, @len);
    Move(Buffer^, password[1], len);
    showmessage(password);

    {write changed buffer to destination file}
//    fsOut.Write(Buffer^, len)
  finally
    {release memory allocated for buffer}
    FreeMem(Buffer, 512);
  end;

  {release the context for crypt default provider}
  CryptReleaseContext(hProv, 0);
exit;

  ShellExecute(0, 'open', 'http://www.scalabium.com/smorder.htm', nil, nil, SW_SHOWNORMAL);
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  Application.MessageBox('TMSWordDocument component, written by Scalabium Software'#13#10 +
                         ''#13#10 +
                         'Allow to read documents created in MS Word without MS Word installed'#13#10 +
                         'Direct and native document processing allow to extract a text very-very fast'#13#10 +
                         ''#13#10 +
                         'For additional tech information or support ask Mike Shkolnik:' + #13#10 +
                         'http://www.scalabium.com, e-mail: mshkolnik@scalabium.com' + #13#10#13#10 +
                         'You can order a product at:  http://www.scalabium.com/smorder.htm'#13#10,
                         'About TMSWordDocument v1.31', MB_OK);
end;

procedure TfrmMain.MSWordDocument1ConvertSysChar(Sender: TObject;
  SysChar: TSMWordSysChar; Value: WideString; var Result: WideString);
var
  i: Integer;
begin
  if (SysChar = scColumnEnd) then
    Result := Result + #13#10//#9
  else
  if (SysChar in [scRowEnd]) then
    Result := Result + #13#10
  else
  if (SysChar in [scHyperlink, scIncludePicture]) then
  begin
    {remove url, leave text only}
    i := Pos(#$14, Result);
    if (i > 0) then
      Delete(Result, 1, i)

    {to leave the url+text, use the next code instead:
{    i := Pos(#$14, Result);
    if (i > 0) then
      Result[i] := ' '
}
  end
  else
  if (SysChar = scFieldEnd) then
  begin
    {remove url, leave text only}
    i := Pos(#$14, Result);
    if (i > 0) then
      Delete(Result, 1, i)
  end
  else
  if (SysChar = scEndOfPage) then
  begin
    Result := '---------------------------------------------------'#13#10
  end
end;

procedure TfrmMain.MSWordDocument1GetDefaultLanguage(Sender: TObject;
  var DocLang: Word);
begin
//  DocLang := LANG_RUSSIAN
end;

end.
